"""APC Power Distribution Unit control object."""

import logging
from datetime import datetime
from os import getcwd, path

from retrying import retry, RetryError
from snmpy import Snmpy


class APC(object):
    """APC Power Distribution Unit."""

    # pylint: disable=too-many-instance-attributes
    # pylint: disable=too-many-public-methods
    # This class requires more attributes and public methods to cover the
    # functionality of the device.

    SNMP_TIMEOUT = 1.5  # 1.5 seconds
    SNMP_RETRIES = 2

    MAX_STOP_DELAY = 15000  # 15 seconds
    INTER_RETRY_WAIT = 500  # 0.5 seconds

    PREFIX = 'PowerNet-MIB'

    # Static readonly data
    Q_NAME = 'rPDU2IdentName.1'
    Q_LOCATION = 'rPDU2IdentLocation.1'
    Q_HARDWARE_REV = 'rPDU2IdentHardwareRev.1'
    Q_FIRMWARE_REV = 'rPDU2IdentFirmwareRev.1'
    Q_MANUFACTURE_DATE = 'rPDU2IdentDateOfManufacture.1'
    Q_MODEL_NUMBER = 'rPDU2IdentModelNumber.1'
    Q_SERIAL_NUMBER = 'rPDU2IdentSerialNumber.1'
    Q_NUM_OUTLETS = 'rPDU2DevicePropertiesNumOutlets.1'
    Q_NUM_SWITCHED_OUTLETS = 'rPDU2DevicePropertiesNumSwitchedOutlets.1'
    Q_NUM_METERED_OUTLETS = 'rPDU2DevicePropertiesNumMeteredOutlets.1'
    Q_MAX_CURRENT_RATING = 'rPDU2DevicePropertiesMaxCurrentRating.1'
    Q_PHASE_VOLTAGE = 'rPDU2PhaseStatusVoltage.1'

    # Dynamic readonly data
    Q_PHASE_LOAD_STATE = 'rPDU2PhaseStatusLoadState.1'
    Q_PHASE_CURRENT = 'rPDU2PhaseStatusCurrent.1'
    Q_POWER = 'rPDU2DeviceStatusPower.1'
    Q_SENSOR_TYPE = 'rPDU2SensorTempHumidityStatusType.1'
    Q_SENSOR_NAME = 'rPDU2SensorTempHumidityStatusName.1'
    Q_SENSOR_COMM_STATUS = 'rPDU2SensorTempHumidityStatusCommStatus.1'
    Q_SENSOR_TEMP_F = 'rPDU2SensorTempHumidityStatusTempF.1'
    Q_SENSOR_TEMP_C = 'rPDU2SensorTempHumidityStatusTempC.1'
    Q_SENSOR_TEMP_STATUS = 'rPDU2SensorTempHumidityStatusTempStatus.1'
    Q_SENSOR_HUMIDITY = 'rPDU2SensorTempHumidityStatusRelativeHumidity.1'
    Q_SENSOR_HUMIDITY_STATUS = 'rPDU2SensorTempHumidityStatusHumidityStatus.1'
    Q_OUTLET_NAME = 'rPDU2OutletSwitchedStatusName.{0}'
    Q_OUTLET_STATUS = 'rPDU2OutletSwitchedStatusState.{0}'

    # Dynamic readwrite data
    Q_SENSOR_NAME_RW = 'rPDU2SensorTempHumidityConfigName.1'
    Q_OUTLET_NAME_RW = 'rPDU2OutletSwitchedConfigName.{0}'
    Q_OUTLET_COMMAND_RW = 'rPDU2OutletSwitchedControlCommand.{0}'

    # Lookups
    LOAD_STATES = [
        '',
        'lowLoad',
        'normal',
        'nearOverload',
        'overload',
    ]
    SENSOR_TYPES = [
        '',
        'temperatureOnly',
        'temperatureHumidity',
        'commsLost',
        'notInstalled',
    ]
    COMM_STATUS_TYPES = [
        '',
        'notInstalled',
        'commsOK',
        'commsLost',
    ]
    SENSOR_STATUS_TYPES = [
        '',
        'notPresent',
        'belowMin',
        'belowLow',
        'normal',
        'aboveHigh',
        'aboveMax',
    ]
    OUTLET_STATUS_TYPES = [
        '',
        'off',
        'on',
    ]

    def _get_query_string(self, query, param=None):
        """Generate a well-formatted SNMP query string.

        :param query: which query to use
        :param param: additional parameter, usually outlet number
        :return: well-formatted SNMP query string
        """
        if param:
            query = query.format(param)
        return '{0}::{1}'.format(self.PREFIX, query)

    def __init__(self, hostname_or_ip_address,
                 public_community, private_community):
        """Create an APC object.

        :param hostname_or_ip_address: hostname or ip address of PDU
        :param public_community: public community string
        :param private_community: private community string
        :return: APC object
        """
        self._host = hostname_or_ip_address
        self._vendor = 'APC'

        self._snmp_public_auth = public_community
        self._snmp_private_auth = private_community

        self._connection = Snmpy(
            self._host, self._snmp_public_auth, self._snmp_private_auth,
            timeout=self.SNMP_TIMEOUT, retries=self.SNMP_RETRIES
        )
        self._connection.add_mib_path(getcwd())
        self._connection.add_mib_path(path.dirname(path.abspath(__file__)))
        self._connection.load_mibs(self.PREFIX)

        # Generic information (static)
        self._identification = self._connection.get(
            self._get_query_string(self.Q_NAME)
        )
        self._location = self._connection.get(
            self._get_query_string(self.Q_LOCATION)
        )
        self._hardware_rev = self._connection.get(
            self._get_query_string(self.Q_HARDWARE_REV)
        )
        self._firmware_rev = self._connection.get(
            self._get_query_string(self.Q_FIRMWARE_REV)
        )
        self._manufacture_date = datetime.strptime(
            str(self._connection.get(
                self._get_query_string(self.Q_MANUFACTURE_DATE))), "%m/%d/%Y"
        )
        self._model_number = self._connection.get(
            self._get_query_string(self.Q_MODEL_NUMBER)
        )
        self._serial_number = self._connection.get(
            self._get_query_string(self.Q_SERIAL_NUMBER)
        )

        # Device status (static)
        self._num_outlets = self._connection.get(
            self._get_query_string(self.Q_NUM_OUTLETS)
        )
        self._num_switched_outlets = self._connection.get(
            self._get_query_string(self.Q_NUM_SWITCHED_OUTLETS)
        )
        self._num_metered_outlets = self._connection.get(
            self._get_query_string(self.Q_NUM_METERED_OUTLETS)
        )
        self._max_current = self._connection.get(
            self._get_query_string(self.Q_MAX_CURRENT_RATING)
        )

        # Phase status (static)
        self._power_factor = 100
        self._current_factor = 10
        self._phase_voltage = int(
            self._connection.get(self._get_query_string(self.Q_PHASE_VOLTAGE))
        )

        self._use_centigrade = False

        self._logger = logging.getLogger(__name__)
        self._logger.addHandler(logging.NullHandler())

    @property
    def host(self):
        """Hostname or IP Address of PDU.

        :return: PDU hostname or ip address
        """
        self._logger.info('Host: %s', self._host)
        return self._host

    @property
    def vendor(self):
        """Vendor/Manufacturer of PDU.

        :return: PDU vendor/manufacturer
        """
        self._logger.info('Vendor: %s', self._vendor)
        return self._vendor

    @property
    def identification(self):
        """Identification string.

        :return: PDU identification
        """
        self._logger.info('Identification: %s', self._identification)
        return self._identification

    @property
    def location(self):
        """Location of the PDU.

        :return: PDU location
        """
        self._logger.info('Location: %s', self._location)
        return self._location

    @property
    def hardware_revision(self):
        """Hardware revision.

        :return: PDU hardware revision
        """
        revision = str(self._hardware_rev)
        self._logger.info('Hardware revision: %s', revision)
        return revision

    @property
    def firmware_revision(self):
        """Firmware revision.

        :return: PDU firmware revision
        """
        revision = str(self._firmware_rev)
        self._logger.info('Firmware revision: %s', revision)
        return revision

    @property
    def date_of_manufacture(self):
        """Date of manufacture.

        :return: PDU date of manufacture
        """
        self._logger.info(
            'Date of Manufacture: %s', str(self._manufacture_date)
        )
        return self._manufacture_date

    @property
    def model_number(self):
        """Model number.

        :return: PDU model number
        """
        model = str(self._model_number)
        self._logger.info('Model number: %s', model)
        return model

    @property
    def serial_number(self):
        """Serial number.

        :return: PDU serial number
        """
        serial = str(self._serial_number)
        self._logger.info('Serial number: %s', serial)
        return serial

    @property
    def num_outlets(self):
        """Number of outlets in the PDU.

        :return: total number of outlets in the PDU
        """
        num = int(self._num_outlets)
        self._logger.info('Number of outlets: %d', num)
        return num

    @property
    def num_switched_outlets(self):
        """Number of switched outlets in the PDU.

        :return: number of switched outlets in the PDU
        """
        num = int(self._num_switched_outlets)
        self._logger.info('Number of switched outlets: %d', num)
        return num

    @property
    def num_metered_outlets(self):
        """Number of metered outlets in the PDU.

        :return: number of metered outlets in the PDU
        """
        num = int(self._num_metered_outlets)
        self._logger.info('Number of metered outlets: %d', num)
        return num

    @property
    def max_current(self):
        """Maximum current for the PDU.

        :return: maximum current for the PDU
        """
        current = int(self._max_current)
        self._logger.info('Maximum current: %d', current)
        return current

    @property
    def voltage(self):
        """Line voltage of the PDU.

        :return: PDU line voltage
        """
        voltage = int(self._phase_voltage)
        self._logger.info('Line voltage: %d', voltage)
        return voltage

    @property
    def load_state(self):
        """Load state of the PDU.

        :return: one of ['lowLoad', 'normal', 'nearOverload', overload']
        """
        state = int(
            self._connection.get(
                self._get_query_string(self.Q_PHASE_LOAD_STATE)
            )
        )
        self._logger.info('Load state: %s', self.LOAD_STATES[state])
        return self.LOAD_STATES[state]

    @property
    def current(self):
        """The current utilization of the PDU.

        :return: current, in amps
        """
        current = float(
            self._connection.get(
                self._get_query_string(self.Q_PHASE_CURRENT)
            ) / self._current_factor
        )
        self._logger.info('Current: %.2f', current)
        return current

    @property
    def power(self):
        """The power utilization of the PDU.

        :return: power, in kW
        """
        power = float(
            self._connection.get(
                self._get_query_string(self.Q_POWER)) / self._power_factor
        )
        self._logger.info('Power: %.2f', power)
        return power

    @property
    def is_sensor_present(self):
        """Determine if a sensor is present on the PDU.

        :return: Is the sensor present?
        """
        state = self._connection.get(
            self._get_query_string(self.Q_SENSOR_TYPE)
        )
        present = 1 < int(state) < 3
        self._logger.info('Sensor present: %s', str(present))
        return present

    @property
    def sensor_name(self):
        """Name of the sensor.

        :return: name of the sensor
        """
        name = None
        if self.is_sensor_present:
            name = str(
                self._connection.get(
                    self._get_query_string(self.Q_SENSOR_NAME)
                )
            )
        self._logger.info('Sensor name: %s', name)
        return name

    @sensor_name.setter
    def _set_sensor_name(self, name):
        """Name of the sensor.

        :param name: name of the sensor
        :return:
        """
        if self.is_sensor_present:
            self._connection.set(
                self._get_query_string(self.Q_SENSOR_NAME_RW), name
            )
            self._logger.info('Updating sensor name to: %s', name)

    @property
    def sensor_type(self):
        """Type of sensor.

        :return: type of sensor, one of
            ['temperatureOnly', 'temperatureHumidity', 'commsLost',
            'notInstalled']
        """
        index = 4
        if self.is_sensor_present:
            index = int(
                self._connection.get(
                    self._get_query_string(self.Q_SENSOR_TYPE)
                )
            )
        self._logger.info('Sensor type: %s', self.SENSOR_TYPES[index])
        return self.SENSOR_TYPES[index]

    @property
    def sensor_comm_status(self):
        """Communication status of the sensor.

        :return: communication status of the sensor
        """
        index = 1
        if self.is_sensor_present:
            index = int(
                self._connection.get(
                    self._get_query_string(self.Q_SENSOR_COMM_STATUS)
                )
            )
        self._logger.info('Sensor communication status: %s',
                          self.SENSOR_STATUS_TYPES[index])
        return self.SENSOR_STATUS_TYPES[index]

    @property
    def use_centigrade(self):
        """Select between centigrade and fahrenheit.

        :return: using centigrade or not
        """
        self._logger.info('Use centigrade: %s', str(self._use_centigrade))
        return self._use_centigrade

    @use_centigrade.setter
    def use_centigrade(self, value):
        """Select between centigrade and fahrenheit.

        :param value: use centrigrade or not
        :return:
        """
        self._logger.info('Updating use centigrade to: %s', value)
        self._use_centigrade = value

    @property
    def temperature(self):
        """Temperature.

        :return: temperature
        """
        temp = 0.00
        if self.sensor_supports_temperature:
            if self._use_centigrade:
                temp = float(
                    self._connection.get(
                        self._get_query_string(self.Q_SENSOR_TEMP_C)
                    ) / 10
                )
            else:
                temp = float(
                    self._connection.get(
                        self._get_query_string(self.Q_SENSOR_TEMP_F)
                    ) / 10
                )
        self._logger.info('Temperature: %.2f', temp)
        return temp

    @property
    def humidity(self):
        """Relative humidity.

        :return: relative humidity
        """
        humid = 0.00
        if self.sensor_supports_humidity:
            humid = float(
                self._connection.get(
                    self._get_query_string(self.Q_SENSOR_HUMIDITY)
                )
            )
        self._logger.info('Relative humidity: %.2f', humid)
        return humid

    @property
    def temperature_status(self):
        """Determine the status of the temperature sensor.

        :return: The status of the temperature sensor
        """
        index = 1
        if self.sensor_supports_temperature:
            index = self._connection.get(
                self._get_query_string(self.Q_SENSOR_TEMP_STATUS)
            )
        self._logger.info(
            'Temperature sensor status: %s', self.SENSOR_STATUS_TYPES[index]
        )
        return self.SENSOR_STATUS_TYPES[index]

    @property
    def humidity_status(self):
        """Determine the status of the humidity sensor.

        :return: status of the humidity sensor
        """
        index = 1
        if self.sensor_supports_humidity:
            index = self._connection.get(
                self._get_query_string(self.Q_SENSOR_HUMIDITY_STATUS)
            )
        self._logger.info(
            'Relative humidity sensor status: %s',
            self.SENSOR_STATUS_TYPES[index]
        )
        return self.SENSOR_STATUS_TYPES[index]

    def get_outlet_name(self, outlet):
        """Name of an outlet in the PDU.

        :param outlet: outlet number
        :return: name of the outlet
        """
        if 1 <= outlet <= self._num_outlets:
            name = str(
                self._connection.get(
                    self._get_query_string(self.Q_OUTLET_NAME, outlet)
                )
            )
            self._logger.info('Outlet number %d has name %s', outlet, name)
            return name
        else:
            raise IndexError(
                'Only %d outlets exist. "%s" is an invalid outlet.',
                self._num_outlets, str(outlet)
            )

    def set_outlet_name(self, outlet, name):
        """Update the name of an outlet in the PDU.

        :param outlet: outlet number
        :param name: outlet name
        :return:
        """
        if 1 <= outlet <= self._num_outlets:
            self._connection.set(
                self._get_query_string(self.Q_OUTLET_NAME_RW, outlet), name
            )
            self._logger.info(
                'Updating outlet number %d to new name %s', outlet, name
            )
        else:
            raise IndexError(
                'Only %d outlets exist. "%s" is an invalid outlet.',
                self._num_outlets, str(outlet)
            )

    def outlet_status(self, outlet):
        """Determine the status of the outlet in the PDU.

        :param outlet: outlet number
        :return: status of the outlet, one of ['on', 'off']
        """
        if 1 <= outlet <= self._num_outlets:
            state = self._connection.get(
                self._get_query_string(self.Q_OUTLET_STATUS, outlet)
            )
            self._logger.info(
                'Outlet number %d has status %s', outlet,
                self.OUTLET_STATUS_TYPES[state]
            )
            return self.OUTLET_STATUS_TYPES[state]
        else:
            raise IndexError(
                'Only %d outlets exist. "%s" is an invalid outlet.',
                self._num_outlets, str(outlet)
            )

    def outlet_command(self, outlet, operation):
        """Send command to an outlet in the PDU.

        :param outlet: outlet number
        :param operation: one of ['on', 'off', 'reboot']
        :return: did the operation complete successfully?
        """
        valid_operations = ['on', 'off', 'reboot']
        if operation not in valid_operations:
            raise ValueError(
                '"%s" is an invalid operation. Valid operations are: %s',
                str(operation), str(valid_operations)
            )

        operations = {
            'on': 1,
            'off': 2,
            'reboot': 3,
        }

        if 1 <= outlet <= self._num_outlets:
            self._logger.info(
                'Setting outlet %d to %s state', outlet, operation
            )
            self._connection.set(
                self._get_query_string(self.Q_OUTLET_COMMAND_RW, outlet),
                operations[operation]
            )

            try:
                if operation in ('on', 'reboot'):
                    success = self.__wait_for_state(outlet, 'on')
                else:
                    success = self.__wait_for_state(outlet, 'off')
            except RetryError:
                # If the operation timed out, no determination of the result
                # can be made.
                success = False

            return success
        else:
            raise IndexError(
                'Only %d outlets exist. "%s" is an invalid outlet.',
                self._num_outlets, str(outlet)
            )

    @property
    def sensor_supports_temperature(self):
        """Determine if the sensor supports temperature measurements.

        :return: does the sensor support temperature measurements?
        """
        return self.is_sensor_present and 'temp' in self.sensor_type.lower()

    @property
    def sensor_supports_humidity(self):
        """Determine if the sensor supports relative humidity measurements.

        :return: does the sensor support relative humidity measurements?
        """
        return self.is_sensor_present and 'humid' in self.sensor_type.lower()

    # pylint: disable=no-self-argument
    # In order to use this method within the @retry decorator, this method
    # must be defined as such.
    def __retry_if_not_state(result):
        """Only keep retrying if the state is not what is expected.

        :return: negation of input
        """
        return not result

    @retry(
        stop_max_delay=MAX_STOP_DELAY, wait_fixed=INTER_RETRY_WAIT,
        retry_on_result=__retry_if_not_state
    )
    def __wait_for_state(self, outlet, state):
        """Wait until state is hit.

        This will wait for MAX_STOP_DELAY with a inter-try delay of
        INTER_RETRY_WAIT.

        :param outlet: outlet number
        :param state: state to wait for
        :return: was the state hit?
        """
        return self.outlet_status(outlet) is state



"""Cisco Switch control object."""

import logging

from netaddr import EUI, mac_unix_expanded
from paramiko import SSHClient, AutoAddPolicy


class CiscoSwitch(object):
    """Cisco switch control."""

    MAX_COMMAND_READ = 16

    CMD_LOGIN_SIGNALS = ['>', '#']
    CMD_GENERIC_SIGNALS = ['#']

    CMD_ENABLE = 'enable'
    CMD_ENABLE_SIGNALS = ['Password']

    CMD_VERSION = 'sh version'
    CMD_VERSION_SIGNALS = ['BOOTLDR']

    CMD_TERMINAL_LENGTH = 'terminal length 0'

    CMD_IPDT = 'sh ip device track all'
    CMD_IPDT_SIGNALS = ['Enabled interfaces']

    CMD_MAC_ADDRESS_TABLE = 'sh mac address-table'
    CMD_MAC_ADDRESS_TABLE_SIGNALS = ['Total Mac Addresses']

    CMD_CONFIGURE = 'configure terminal'
    CMD_CONFIGURE_INTERFACE = 'int {0}'
    CMD_CONFIGURE_SIGNALS = ['(config)#', '(config-if)#']

    CMD_POWER_OFF = 'power inline never'
    CMD_POWER_ON = 'power inline auto'

    CMD_VLAN_MODE_ACCESS = 'switchport mode access'
    CMD_VLAN_SET = 'switchport access vlan {0}'

    CMD_END = 'end'

    PORT_NOTATION = {
        'fastethernet': 'Fa',
        'gigabitethernet': 'Gi',
        'tengigabitethernet': 'Ten',
    }

    def __init__(self, hostname_or_ip_address, username, password):
        """Initialize the CiscoSwitch object.

        :param hostname_or_ip_address: host name or ip address
        :param username: username to login with
        :param password: password to use with username
        :return: CiscoSwitch object
        """
        self._host = hostname_or_ip_address
        self._version = None
        self._username = username
        self._password = password
        self._client = None
        self._shell = None
        self._enable_needed = False
        self._ready = False

        self._logger = logging.getLogger(__name__)
        self._logger.addHandler(logging.NullHandler())

    def connect(self):
        """Connect to the switch.

        :return:
        """
        self._client = SSHClient()
        self._client.set_missing_host_key_policy(AutoAddPolicy())
        self._client.connect(self._host,
                             username=self._username,
                             password=self._password,
                             allow_agent=False,
                             look_for_keys=False)
        self._shell = self._client.invoke_shell()
        self._ready = True
        output = self._send_command('', self.CMD_LOGIN_SIGNALS)
        if output.find('>') > 0:
            self._enable_needed = True
            self._ready = False
        else:
            self.set_terminal_length()

    def disconnect(self):
        """Disconnect from the switch.

        :return:
        """
        if self._client is not None:
            self._client.close()
            self._client = None
            self._shell = None
            self._ready = False

    def enable(self, password):
        """Put the switch in enable mode.

        :param password: password to use to enable
        :return:
        """
        if self.connected and self._enable_needed:
            self._send_command(self.CMD_ENABLE, self.CMD_ENABLE_SIGNALS)
            self._send_command(
                password, self.CMD_GENERIC_SIGNALS, log=False
            )
            self._ready = True
            self.set_terminal_length()

    def set_terminal_length(self):
        """Set terminal length.

        :return:
        """
        if not self._ready:
            return

        self._send_command(self.CMD_TERMINAL_LENGTH, self.CMD_GENERIC_SIGNALS)

    def ipdt(self):
        """IP Device Tracking (IPDT) information.

        :return: IPDT information
        """
        if self._ready:
            output = self._send_command(self.CMD_IPDT, self.CMD_IPDT_SIGNALS)
            return CiscoSwitch._parse_ipdt_output(output)
        return None

    def mac_address_table(self, ignore_port=None):
        """MAC Address Table Information.

        :param ignore_port: port to ignore, e.g. Gi1/0/48
        :return: ARP information
        """
        if not self._ready:
            return None

        output = self._send_command(
            self.CMD_MAC_ADDRESS_TABLE, self.CMD_MAC_ADDRESS_TABLE_SIGNALS
        )
        return self._parse_mac_address_table_output(
            output, ignore_port=ignore_port
        )

    def poe_on(self, port):
        """Enable a port for POE.

        :param port: port to enable POE on, e.g. Gi1/0/1
        :return:
        """
        if not self._ready:
            return

        port = self._shorthand_port_notation(port)
        cmds = [
            (self.CMD_CONFIGURE, self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_CONFIGURE_INTERFACE.format(port),
             self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_POWER_ON, self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_END, self.CMD_GENERIC_SIGNALS)
        ]
        self._send_commands(cmds)

    def poe_off(self, port):
        """Disable a port for POE.

        :param port: port to disable POE on, e.g. Gi1/0/1
        :return:
        """
        if not self._ready:
            return

        port = self._shorthand_port_notation(port)
        cmds = [
            (self.CMD_CONFIGURE, self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_CONFIGURE_INTERFACE.format(port),
             self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_POWER_OFF, self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_END, self.CMD_GENERIC_SIGNALS)
        ]
        self._send_commands(cmds)

    def change_vlan(self, port, vlan):
        """Change the VLAN assignment on a port.

        :param port: port to change VLAN assignment on, e.g. Gi1/0/1
        :param vlan: VLAN id
        :return:
        """
        if not self._ready:
            return

        port = self._shorthand_port_notation(port)
        cmds = [
            (self.CMD_CONFIGURE, self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_CONFIGURE_INTERFACE.format(port),
             self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_VLAN_MODE_ACCESS, self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_VLAN_SET.format(vlan), self.CMD_CONFIGURE_SIGNALS),
            (self.CMD_END, self.CMD_GENERIC_SIGNALS)
        ]
        self._send_commands(cmds)

    @property
    def version(self):
        """The Cisco IOS version.

        :return: The Cisco IOS version.
        """
        if self._version is None:
            if self._ready:
                output = self._send_command(
                    self.CMD_VERSION, self.CMD_VERSION_SIGNALS
                )
                self._version = self._parse_version_output(output)
        return self._version

    @property
    def host(self):
        """The IP address or hostname of the switch.

        :return: IP address or hostname of the switch
        """
        return self._host

    @property
    def connected(self):
        """Switch connection status.

        :return: has a connection to the switch been made?
        """
        return self._shell is not None

    def _shorthand_port_notation(self, port):
        """Shorthand port notation.

        Takes a port name, such as Gi1/0/48 or GigabitEthernet1/0/48 and
        returns the shorthand notation.

        :param port: port name
        :return: shorthand port name
        """
        if not port:
            return

        lower = port.lower()
        output = port

        if any(lower.find(port) == 0 for port in self.PORT_NOTATION.keys()):
            for item in self.PORT_NOTATION.items():
                if lower.startswith(item[0]):
                    output = lower.replace(item[0], item[1])
                    break

        return output

    def _send_commands(self, commands):
        """Send a list of commands to the SSH shell.

        :param commands: list of commands
        """
        for command in commands:
            self._send_command(command[0], command[1])

    def _send_command(self, command, signals, log=True):
        """Send a command to the SSH shell.

        Sends a command to the SSH shell and waits for the signals to arrive.

        :param command: command to send
        :param signals: signals to wait for
        :param log: specifies if this command should be logged
        :return: output of the command
        """
        send_command = command + '\n'
        if log:
            self._logger.info('Sending command: %s', send_command)

        self._shell.send(send_command)

        read_buffer = ''
        while not any(read_buffer.find(signal) > -1 for signal in signals):
            read_buffer += self._shell.recv(self.MAX_COMMAND_READ)

        if log:
            self._logger.debug('Received output: %s', read_buffer)

        # Manually flush the rest of the buffer.
        while self._shell.recv_ready():
            self._shell.recv(self.MAX_COMMAND_READ)

        return read_buffer

    @staticmethod
    def _parse_version_output(output):
        """Version parsing.

        :param output: the output of the command
        :return: version string
        """
        lines = [line.strip() for line in output.splitlines()]
        version = ''
        search = True
        while len(lines) > 0 and search:
            line = lines.pop()

            if line.find('Cisco IOS') < 0:
                continue

            version_info = line.split(',')

            if len(version_info) > 1:
                version = version_info[2].strip()
                version = version[8:]
                search = False

        return version

    def _parse_mac_address_table_output(self, output, ignore_port=None):
        """Mac Address Table parsing.

        Parses the output of 'sh mac address-table' command.

        :param output: the output of the command
                      Mac Address Table
            -------------------------------------------
            Vlan    Mac Address       Type        Ports
            ----    -----------       --------    -----
             All    0100.0ccc.cccc    STATIC      CPU
             <snip>
             All    ffff.ffff.ffff    STATIC      CPU
             601    000b.7866.5240    DYNAMIC     Gi1/0/48
             601    0013.20fe.56b4    DYNAMIC     Gi1/0/35
             <snip>
             Total Mac Addresses for this criterion: 59
        :param ignore_port: ignore this port
        :return: list of dicts containing device connection metrics
        """
        lookup = []
        lines = [line.strip() for line in output.splitlines()]
        lines.append('')
        while len(lines) > 0:
            line = lines.pop()

            # Table entries will always have a '.' for the MAC address.
            # If there isn't one it's not a row we care about.
            if line.find('.') < 0:
                continue

            values = [entry for entry in line.split() if entry]

            # Ignore non-physical ports
            port_types = self.PORT_NOTATION.values()
            if all(values[3].lower().find(port.lower()) != 0
                   for port in port_types):
                continue

            # If the ignore_port is specified and is the port in question,
            # ignore it.
            ignore_port = self._shorthand_port_notation(ignore_port)
            if ignore_port and values[3] == ignore_port:
                continue

            lookup.append(
                {
                    'mac': EUI(values[1], dialect=mac_unix_expanded),
                    'interface': values[3]
                }
            )
        return sorted(lookup, key=lambda k: k['interface'])

    @staticmethod
    def _parse_ipdt_output(output):
        """IPDT output parsing.

        Parses the output of the `show ip device track` command.

        :param output: the output of the command
            IP Device Tracking = Enabled
            IP Device Tracking Probe Count = 3
            IP Device Tracking Probe Interval = 30
            IP Device Tracking Probe Delay Interval = 0
            -------------------------------------------------------------------
              IP Address     MAC Address   Vlan  Interface              STATE
            -------------------------------------------------------------------
            192.168.1.12     6cec.eb68.c86f  601  GigabitEthernet1/0/14  ACTIVE
            192.168.1.15     6cec.eb67.836c  601  GigabitEthernet1/0/12  ACTIVE
            <snip>

            Total number interfaces enabled: 47
            Enabled interfaces:
             Gi1/0/1, Gi1/0/2, Gi1/0/3, Gi1/0/4, Gi1/0/5, Gi1/0/6, Gi1/0/7,
             <snip>
        :return: list of dicts containing device connection metrics
        """
        lookup = []
        lines = [line.strip() for line in output.splitlines()]
        lines.append('')
        while len(lines) > 0:
            line = lines.pop()

            # Table entries will always have a '.' for the MAC address.
            # If there isn't one it's not a row we care about.
            if line.find('.') < 0:
                continue

            values = [entry for entry in line.split() if entry]

            # Ignore any 'INACTIVE' entries, meaning that there hasn't been
            # traffic from that MAC Address is has likely been unplugged.
            if values[4] == 'INACTIVE':
                continue

            lookup.append(
                {
                    'ip': values[0],
                    'mac': EUI(values[1], dialect=mac_unix_expanded),
                    'interface': values[3]
                }
            )
        return sorted(lookup, key=lambda k: k['interface'])



"""Initialization of the pynoc package."""

# flake8: noqa
# pylint: disable=unused-import
# Since this is the package init, the imports here are not used within
# this file.

from pkg_resources import get_distribution

from .cisco import CiscoSwitch
from .apc import APC

# pylint: disable=maybe-no-member
# False positives: This message may report object members that are
# created dynamically, but exist at the time they are accessed.
__version__ = get_distribution('pynoc').version



import os
import unittest
from pynoc import CiscoSwitch

ENV_NOT_SET = "Please set environment variable: {0}."
TEST_ENV_IP_ADDRESS = "CISCO_IP_ADDRESS"
TEST_ENV_USERNAME = "CISCO_USERNAME"
TEST_ENV_PASSWORD = "CISCO_PASSWORD"
TEST_ENV_ENABLE_PASSWORD = "CISCO_ENABLE_PASSWORD"


class TestCisco(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cisco_address = os.getenv(TEST_ENV_IP_ADDRESS)
        if cisco_address is None:
            raise EnvironmentError(ENV_NOT_SET.format(TEST_ENV_IP_ADDRESS))
        cisco_username = os.getenv(TEST_ENV_USERNAME)
        if cisco_username is None:
            raise EnvironmentError(ENV_NOT_SET.format(TEST_ENV_USERNAME))
        cisco_password = os.getenv(TEST_ENV_PASSWORD)
        if cisco_password is None:
            raise EnvironmentError(ENV_NOT_SET.format(TEST_ENV_PASSWORD))
        cisco_enable_password = os.getenv(TEST_ENV_ENABLE_PASSWORD)
        if cisco_address is None:
            raise EnvironmentError(
                ENV_NOT_SET.format(TEST_ENV_ENABLE_PASSWORD))
        cls.cisco_enable_password = cisco_enable_password
        cls.cisco = CiscoSwitch(cisco_address, cisco_username, cisco_password)

    @classmethod
    def tearDownClass(cls):
        if hasattr(cls, "cisco") and cls.cisco is not None:
            del cls.cisco

    def test_0_connect(self):
        self.assertFalse(self.cisco.connected)
        self.cisco.connect()
        self.assertTrue(self.cisco.connected)

    def test_1_enable(self):
        self.cisco.enable(self.cisco_enable_password)

    def test_2_set_terminal_length(self):
        self.cisco.set_terminal_length()

    def test_ipdt(self):
        self.assertIsNotNone(self.cisco.ipdt())

    def test_mac_address_table(self):
        self.assertIsNotNone(self.cisco.mac_address_table())

    def test_host(self):
        self.assertIsNotNone(self.cisco.host)

    def test_shorthand_notation(self):
        shorthand = self.cisco._shorthand_port_notation("FastEthernet1/0/1")
        self.assertEqual(shorthand, "Fa1/0/1")
        shorthand = self.cisco._shorthand_port_notation("GigabitEthernet1/0/1")
        self.assertEqual(shorthand, "Gi1/0/1")
        shorthand = self.cisco._shorthand_port_notation("TenGigabitEthernet1/0/1")
        self.assertEqual(shorthand, "Ten1/0/1")

    def test_zzz_disconnect(self):
        self.cisco.disconnect()



import os
import unittest
from pynoc import APC

ENV_NOT_SET = "Please set environment variable: {0}."
TEST_ENV_IP_ADDRESS = "APC_IP_ADDRESS"
TEST_ENV_PUBLIC_COMMUNITY = "APC_PUBLIC_COMMUNITY"
TEST_ENV_PRIVATE_COMMUNITY = "APC_PRIVATE_COMMUNITY"


class TestAPC(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        apc_address = os.getenv(TEST_ENV_IP_ADDRESS)
        if apc_address is None:
            raise EnvironmentError(ENV_NOT_SET.format(TEST_ENV_IP_ADDRESS))
        public_community = os.getenv(TEST_ENV_PUBLIC_COMMUNITY)
        if public_community is None:
            raise EnvironmentError(
                ENV_NOT_SET.format(TEST_ENV_PUBLIC_COMMUNITY))
        private_community = os.getenv(TEST_ENV_PRIVATE_COMMUNITY)
        if private_community is None:
            raise EnvironmentError(
                ENV_NOT_SET.format(TEST_ENV_PRIVATE_COMMUNITY))
        cls.apc = APC(apc_address, public_community=public_community,
                      private_community=private_community)

    @classmethod
    def tearDownClass(cls):
        if hasattr(cls, "apc") and cls.apc is not None:
            del cls.apc

    def test_host(self):
        self.assertIsNotNone(self.apc.host)

    def test_vendor(self):
        self.assertIsNotNone(self.apc.vendor)

    def test_identification(self):
        self.assertIsNotNone(self.apc.identification)

    def test_location(self):
        self.assertIsNotNone(self.apc.location)

    def test_hardware_revision(self):
        self.assertIsNotNone(self.apc.hardware_revision)

    def test_firmware_revision(self):
        self.assertIsNotNone(self.apc.firmware_revision)

    def test_date_of_manufacture(self):
        self.assertIsNotNone(self.apc.date_of_manufacture)

    def test_model_number(self):
        self.assertIsNotNone(self.apc.model_number)

    def test_serial_number(self):
        self.assertIsNotNone(self.apc.serial_number)

    def test_num_outlets(self):
        self.assertIsNotNone(self.apc.num_outlets)
        self.assertGreater(self.apc.num_outlets, -1)

    def test_num_switched_outlets(self):
        self.assertIsNotNone(self.apc.num_switched_outlets)
        self.assertGreater(self.apc.num_switched_outlets, -1)

    def test_num_metered_outlets(self):
        self.assertIsNotNone(self.apc.num_metered_outlets)
        self.assertGreater(self.apc.num_metered_outlets, -1)

    def test_max_current(self):
        self.assertIsNotNone(self.apc.max_current)
        self.assertGreater(self.apc.max_current, -1)

    def test_voltage(self):
        self.assertIsNotNone(self.apc.voltage)
        self.assertGreater(self.apc.voltage, -1)

    def test_load_state(self):
        self.assertIsNotNone(self.apc.load_state)
        self.assertIn(self.apc.load_state,
                      ['lowLoad', 'normal', 'nearOverload', 'overload'])

    def test_current(self):
        self.assertIsNotNone(self.apc.current)
        self.assertGreater(self.apc.current, -1)

    def test_power(self):
        self.assertIsNotNone(self.apc.power)
        self.assertGreater(self.apc.power, -1)

    def test_current(self):
        self.assertIsNotNone(self.apc.current)
        self.assertGreater(self.apc.current, -1)

    def test_is_sensor_present(self):
        self.assertIsNotNone(self.apc.is_sensor_present)

    def test_sensor_name(self):
        self.assertIsNotNone(self.apc.sensor_name)

    def test_sensor_type(self):
        self.assertIsNotNone(self.apc.sensor_type)
        self.assertIn(self.apc.sensor_type,
                      ['temperatureOnly', 'temperatureHumidity', 'commsLost',
                       'notInstalled'])

    def test_sensor_comm_status(self):
        self.assertIsNotNone(self.apc.sensor_comm_status)
        self.assertIn(self.apc.sensor_comm_status,
                      ['notPresent', 'belowMin', 'belowLow', 'normal',
                       'aboveHigh', 'aboveMax'])

    def test_temperature(self):
        self.assertIsNotNone(self.apc.temperature)
        self.apc.use_centigrade = True
        self.assertIsNotNone(self.apc.temperature)
        self.apc.use_centigrade = False

    def test_humidity(self):
        self.assertIsNotNone(self.apc.humidity)

    def test_temperature_status(self):
        self.assertIsNotNone(self.apc.temperature_status)
        self.assertIn(self.apc.temperature_status,
                      ['notPresent', 'belowMin', 'belowLow', 'normal',
                       'aboveHigh', 'aboveMax'])

    def test_humidity_status(self):
        self.assertIsNotNone(self.apc.humidity_status)
        self.assertIn(self.apc.humidity_status,
                      ['notPresent', 'belowMin', 'belowLow', 'normal',
                       'aboveHigh', 'aboveMax'])

    def test_get_outlet_name(self):
        for num in range(1, self.apc.num_outlets + 1):
            self.assertIsNotNone(self.apc.get_outlet_name(num))

    def test_outlet_status(self):
        for num in range(1, self.apc.num_outlets + 1):
            self.assertIsNotNone(self.apc.outlet_status(num))
            self.assertIn(self.apc.outlet_status(num),
                          ['off', 'on'])



#!/usr/bin/env python

"""Setup configuration."""

from setuptools import setup, find_packages

setup(
    name='pynoc',
    version='1.3.1',

    description='Network Operation Center gear',
    long_description='Python package to handle interact with various '
                     'network operation center gear. This includes: network '
                     'switches, power distribution units, and such.',
    url='http://github.com/SimplicityGuy/pynoc',

    author='Robert Wlodarczyk',
    author_email='robert@simplicityguy.com',

    license='Apache License 2.0',

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: System :: Hardware',
        'Topic :: System :: Networking',
    ],

    keywords=[
        'instrument',
        'switch',
        'pdu',
        'Cisco',
        'APC',
        'network gear',
    ],

    packages=find_packages(exclude=['contrib', 'docs', 'test*']),

    setup_requires=[
        'setuptools-lint',
        'flake8',
        'sphinx',
    ],

    install_requires=[
        'pysnmp',
        'snmpy',
        'paramiko',
        'netaddr',
        'retrying',
    ],

    test_suite='nose.collector',

    tests_require=[
        'nose',
        'nosexcover',
    ],
)
